import java.util.*
import java.io.*
import org.apache.commons.io.filefilter.WildcardFileFilter
import org.apache.commons.io.comparator.NameFileComparator

final def workDir = new File('.').canonicalFile
final def props = new Properties();
final def inputPropsFile = new File(args[0]);
try {
    inputPropsStream = new FileInputStream(inputPropsFile);
    props.load(inputPropsStream);
}
catch (IOException e) {
    throw new RuntimeException(e);
}

def sqlPlusExecutable = props['sqlPlusExecutable'];
println "Executable : " + sqlPlusExecutable;
def user = props['user'];
println "User : " + user;
def password = props['password'];
def connectionID = props['connectionID'];
println "ConnectionID : " + connectionID;
def unparsedSqlFiles = props['sqlFiles'];
unparsedSqlFiles = unparsedSqlFiles.replace('\n', ",");
def sqlFiles = unparsedSqlFiles.split(',');
println "SQL Files : ";
sqlFiles.each {
    println "\t" + it;
}
def oracleHome = props['oracleHome']
println "Oracle Home: ${oracleHome?:''}"

def existingScripts = new ArrayList<String>()

sqlFiles.each { sqlFileName ->

    if (sqlFileName.contains("*") || sqlFileName.contains("?")) {
        def wildcardSqlFile = new File(sqlFileName)
        def wildcard = wildcardSqlFile.getName()
        FilenameFilter filter = new WildcardFileFilter(wildcard)
        def parentDir = wildcardSqlFile.getParentFile()

        if (parentDir == null) {
            parentDir = workDir
        }

        def matchingFiles = Arrays.asList(parentDir.listFiles((FilenameFilter) filter))
        Comparator<File> nameComparator = NameFileComparator.NAME_COMPARATOR
        Collections.sort(matchingFiles, nameComparator)

        matchingFiles.each { matchingFile ->
            existingScripts.add(matchingFile.getCanonicalPath())
        }

    } else if (! (new File(sqlFileName)).exists()) {
        throw new RuntimeException("SQL File ${sqlFileName} does not exist!")
    } else {
        existingScripts.add(sqlFileName)
    }
}

existingScripts.each { existingFileName ->
    def cmdArgs = [sqlPlusExecutable, "${user}/${password}@${connectionID}", '@'+existingFileName];
    println "Executing : " + cmdArgs.join(' ');

    def proc
    if (oracleHome) {
        def env = []
        System.getenv().each {
            env << it.key + '=' + it.value
        }
        env << "ORACLE_HOME=$oracleHome"
        proc = cmdArgs.execute(env, workDir)
    }
    else {
        proc = cmdArgs.execute()
    }

    proc.withWriter { writer ->
       writer.write("exit\n");
    }

    proc.waitForProcessOutput(System.out, System.out);
    proc.waitFor();
    if (proc.exitValue() != 0) {
        throw new RuntimeException("Process Failed!");
    }
}
System.exit(0);

